// Player demo with buzz.js

var pl = {};

djrandom.EventObject = function() {
  var events = {};

  this.emit = function(event) {
    if (!events[event]) return;

    var args = Array.prototype.slice.call(arguments, 1),
    callbacks = events[event];

    for (var i = 0, len = callbacks.length; i < len; i++) {
      callbacks[i].apply(null, args);
    }
  };

  this.on = function(event, fn) {
    events[event] || (events[event] = []);
    events[event].push(fn);
  };

  this.off = function(event, fn) {
    var eventsOf = events[event],
    index = eventsOf.indexOf(fn);

    ~index && eventsOf.splice(index, 1);
  };
};


djrandom.PlayerUI = function() {

  var state = 'paused',
      el = $('.player .button'),
      ev = new djrandom.EventObject();

  function updateui() {
    if (state == 'playing') {
      el.toggleClass('pause', true);
    } else {
      el.toggleClass('pause', false);
    }
  }

  function toggleState() {
    if (state == 'playing') {
      state = 'paused';
      ev.emit('pause');
    } else {
      state = 'playing';
      ev.emit('play');
    }
    updateui();
  }

  el.click(function() {
    toggleState();
  });

  this.SetState = function(playing) {
    if (playing) {
      state = 'playing';
    } else {
      state = 'paused';
    }
    updateui();
  };

  this.GetState = function() {
    return state;
  };

  this.on = function(event, fn) {
    ev.on(event, fn);
  };
};

djrandom.Player = function() {

  var sound = null,
      playing = false,
      ev = new djrandom.EventObject();

  // Play a new song.
  this.Play = function(song) {
    console.log('playing(' + song.id + ')...');
    var url = '/dl/' + song.audio_file.md5;
    newSound = new buzz.sound(url, {preload: true});
    ev.emit('loading', song);

    // 'next' is raised when a song ends.
    newSound.bind('ended', function(e) {
      ev.emit('next');
    });
    // 'error' is raised on errors.
    newSound.bind('error', function(e) {
      console.log('Error: ' + this.getErrorMessage());
      ev.emit('pause');
      ev.emit('error');
      this.stop();
    });
    // 'pause' is raised when no audio is being output. It is a UI signal.
    newSound.bind('ended pause', function(e) {
      // emit the pause signal to the UI
      console.log('ended/pause');
      ev.emit('pause');
    });
    // 'play' is raised when audio is playing. It is a UI signal.
    newSound.bind('playing', function(e) {
      // emit the play signal to the UI
      console.log('play');
      ev.emit('play');
    });

    // Play the new sound, possibly with a faded transition.
    ev.emit('playing', song);
    if (sound && !sound.isEnded() && !sound.isPaused()) {
      sound.unbind('ended pause');
      sound.fadeWith(newSound.play(), 200);
    } else {
      newSound.play().fadeIn(200);
    }

    sound = newSound;
  };

  // Pauses *and* unpauses the currently playing sound.
  this.Pause = function() {
    if (sound) {
      sound.togglePlay();
    }
  };

  // Stop the current sound and destroy it.
  this.Stop = function() {
    if (sound) {
      console.log('stopping...');
      sound.stop();
      sound = null;
    }
  };

  this.on = function(event, fn) {
    ev.on(event, fn);
  };

};

// Sort an array using a key function.
// The key function is only called once for every input item (so you
// can do things like return a random number to obtain a randomly
// ordered list, etc).
function sortBy(list, keyfn) {
    var keys = [];
    for (var i = 0; i < list.length; i++) {
        keys.push({index: i, key: keyfn(list[i])});
    }
    keys.sort(function(a, b) {
        return (a.key>b.key) ? 1 : (a.key<b.key) ? -1 : 0;
    });
    var result = [];
    for (var i = 0; i < keys.length; i++) {
        result.push(list[keys[i].index]);
    }
    return result;
}

